/*
 * samma
 *
 * Copyright (C) 2006,2007,2008,2011 DesigNET, INC.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

/*
 * $RCSfile: mailzip.c,v $
 * $Revision: 1.22 $
 * $Date: 2011/12/28 03:22:21 $
 */

#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <limits.h>
#include <string.h>
#include <pthread.h>
#include <libmilter/mfapi.h>
#include <libdgmail.h>
#include <gmime/gmime.h>
#include <signal.h>
#include <time.h>

#include "log.h"
#include "mailzip.h"
#include "mailzip_db.h"
#include "mailzip_config.h"
#include "client_side.h"
#include "maildrop.h"
#include "mailsave.h"
#include "zipconv.h"
#include "samma_policy.h"
#include "samma_autobcc.h"
#include "global.h"

#define MLFIPRIV(ctx)        ((struct mlfiPriv *) smfi_getpriv(ctx))

/*
 *  sfsistat mlfi_connect
 */
sfsistat
mlfi_connect(SMFICTX *ctx, char *hostname, _SOCK_ADDR *addr)
{
    struct mlfiPriv *priv;
    struct mailinfo *minfo;

    DEBUGLOG("mlfi_connect start");

    /* mail save */
    priv = malloc(sizeof *priv);
    if(priv == NULL) {
	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "priv", strerror(errno));
        return SMFIS_TEMPFAIL;
    }

    memset(priv, 0, sizeof *priv);

    /* read config */
    priv->mlfi_conf = config_init();

    /* set RcptTo check flag */
    priv->mlfi_rcptcheck = BEFOR_RCPTCHECK;

    /* set encription flag */
    priv->mlfi_encstatus = ENC;

    /* set AutoBcc check flag */
    if (priv->mlfi_conf->cf_autobccoption[0] == 'Y' || priv->mlfi_conf->cf_autobccoption[0] == 'y') {
	priv->mlfi_bccoption = ENABLE_AUTOBCC;
    }
    else {
	priv->mlfi_bccoption = DISABLE_AUTOBCC;
    }

    /* create mailinfo structure */
    minfo = malloc(sizeof *minfo);
    if(minfo == NULL) {
	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo", strerror(errno));
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }
    memset(minfo, 0, sizeof *minfo);

    /* allocate message buffer */
    minfo->ii_mbuf = (char *)malloc(MBSIZE + 1);
    if(minfo->ii_mbuf == NULL) {
	log(ERR_MEMORY_ALLOCATE, "mlfi_connect", "minfo->ii_mbuf", strerror(errno));
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }
    
    /* Initialize mailinfo structure */
    minfo->ii_bufsize = MBSIZE;

    priv->mlfi_minfo = minfo;

    /* copy buffer */
    smfi_setpriv(ctx, priv);

    return SMFIS_CONTINUE;
}

/*
 *  sfsistat mlfi_helo
 */
sfsistat
mlfi_helo(SMFICTX *ctx, char *helohost)
{
    DEBUGLOG("mlfi_helo start");

    return SMFIS_CONTINUE;
}

/*
 *  sfsistat mlfi_envfrom
 *  env
 */
sfsistat
mlfi_envfrom(SMFICTX *ctx, char **envfrom)
{
    char *fromaddr;
    int ret;
    struct mlfiPriv *priv;
    char *fqdn;

    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_envfrom");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    DEBUGLOG("mlfi_envfrom start");

    /* get mail addr */
    fromaddr = get_addrpart((unsigned char *)*envfrom);
    if (fromaddr == NULL) {
	log(ERR_MAIL_FIND_ADDRESS, "mlfi_envfrom");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    /* domain null check */
    fqdn = strchr(fromaddr, ATMARK);
    if (fqdn == NULL) {
        free(fromaddr);
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_ACCEPT;
    }

    /* search fromaddr */
    ret = search_fromaddr_bdb(priv->mlfi_conf, fromaddr);
    if (ret < 0) {
        free(fromaddr);
        mlfi_cleanup(ctx, MLFIABORT);

        return SMFIS_TEMPFAIL;
    } else if (ret == NOT_ENC) {
	if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
	    free(fromaddr);
	    mlfi_cleanup(ctx, MLFIABORT);

	    return SMFIS_ACCEPT;
	}
	/* save envription status */
	priv->mlfi_encstatus = NOT_ENC;
    }
    /* save from addr temporally */
    priv->mlfi_savefrom = fromaddr;

    return SMFIS_CONTINUE;
}

/*
 *  sfsistat mlfi_envrcpt
 *  env
 */
sfsistat
mlfi_envrcpt(SMFICTX *ctx, char **rcptto)
{
    char *rcptaddr = NULL;
    struct mlfiPriv *priv;
    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_envrcpt");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    DEBUGLOG("mlfi_envrcpt start");

    /* get mail addr */
    rcptaddr = get_addrpart_notranslate((unsigned char *)*rcptto);
    if (rcptaddr == NULL) {
	if (rcptaddr != NULL) {
            free(rcptaddr);
	}
	log(ERR_MAIL_FIND_ADDRESS, "mlfi_envrcpt");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    /* push rcpt addr */
    if (push_rcptlist(&(priv->mlfi_savercpt), rcptaddr) != 0) {
	mlfi_cleanup(ctx, MLFIABORT);
	free(rcptaddr);
	return SMFIS_TEMPFAIL;
    }
    free(rcptaddr);

    /* set RcptTo check flag */
    priv->mlfi_rcptcheck = BEFOR_RCPTCHECK;

    return SMFIS_CONTINUE;
}

/*
 * sfsistat mlfi_header
 * header
 */
sfsistat
mlfi_header(SMFICTX *ctx, char *headerf, char *headerv)
{
    char *header;
    int ret, len;
    struct mlfiPriv *priv;
    struct rcptinfo *passlist = NULL, *rdmpasslist = NULL;

    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_header");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    DEBUGLOG("mlfi_header start");

    if (priv->mlfi_encstatus == NOT_ENC) {
	return SMFIS_CONTINUE;
    }

    /* RcptTo Check */
    if (priv->mlfi_rcptcheck == BEFOR_RCPTCHECK) {
	if (priv->mlfi_bccoption == ENABLE_AUTOBCC) {
	    ret = add_bccaddr(priv->mlfi_conf, priv->mlfi_savefrom,
			     &priv->mlfi_savercpt, &priv->mlfi_savebcc);
	    if (ret != SUCCESS) {
		mlfi_cleanup(ctx, MLFIABORT);
		return SMFIS_TEMPFAIL;
	    }
	}

	ret = search_rcptaddr(priv->mlfi_conf, priv->mlfi_savefrom, 
			      priv->mlfi_savercpt, &passlist, &rdmpasslist);
	if (ret < 0) {
	    free_rcptinfo(passlist);
	    free_rcptinfo(rdmpasslist);
	    mlfi_cleanup(ctx, MLFIABORT);
	    return SMFIS_TEMPFAIL;
	}
	if (ret == NOT_ENC) {
	    free_rcptinfo(passlist);
	    free_rcptinfo(rdmpasslist);
	    if (priv->mlfi_bccoption == DISABLE_AUTOBCC) {
		mlfi_cleanup(ctx, MLFIABORT);
		return SMFIS_ACCEPT;
	    }
	    /* save envription status */
	    priv->mlfi_encstatus = NOT_ENC;
	    priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
	    return SMFIS_CONTINUE;
	}

	/* save encryption address list */
	priv->mlfi_passlist = passlist;
	priv->mlfi_rdmpasslist = rdmpasslist;

	priv->mlfi_rcptcheck = AFTER_RCPTCHECK;
    }

    /* create header line */
    len = strlen(headerf) + strlen(headerv) + 4;
    header = malloc(len + 1);
    if (!header) {
        log(ERR_MEMORY_ALLOCATE, "mlfi_header", "header", strerror(errno));
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }
    sprintf(header, "%s: %s\n", headerf, headerv);

    /* write header */
    if (mailsave_write(priv->mlfi_minfo, priv->mlfi_conf, header, len - 1) != 0 ) {
        /* write error */
        log(ERR_MAILSAVE_WRITE, "mlfi_header", header);
        free(header);
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }
    free(header);

    return SMFIS_CONTINUE;
}

/*
 * mlfi_eoh
 * header & body
 */
sfsistat
mlfi_eoh(SMFICTX *ctx)
{
    struct mlfiPriv *priv;

    DEBUGLOG("mlfi_eoh start");

    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_eoh");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    if (priv->mlfi_encstatus == NOT_ENC) {
	return SMFIS_CONTINUE;
    }

    /* write new line between header and body */
    if (mailsave_write(priv->mlfi_minfo, priv->mlfi_conf, "\n", 1) == -1 ) {
        /* write error */
        log(ERR_MAILSAVE_WRITE, "mlfi_eoh", "blank line");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    return SMFIS_CONTINUE;
}

/*
 * mlfi_body
 * write body
 */
sfsistat
mlfi_body(SMFICTX *ctx, u_char *bodyp, size_t bodylen)
{
    struct mlfiPriv *priv;
    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_body");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    DEBUGLOG("mlfi_body start");

    if (priv->mlfi_encstatus == NOT_ENC) {
	return SMFIS_CONTINUE;
    }

    /* write body */
    if (mailsave_write(priv->mlfi_minfo, priv->mlfi_conf, (char *)bodyp, bodylen) == -1 ) {
        /* write error */
        log(ERR_MAILSAVE_WRITE, "mlfi_body", "body data");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    return SMFIS_CONTINUE;
}

/*
 * mlfi_eom
 * end
 */
sfsistat
mlfi_eom(SMFICTX *ctx)
{
    int ret, i;
    struct mlfiPriv *priv;
    if ((priv = MLFIPRIV(ctx)) == NULL) {
	log(ERR_GET_PRIV, "mlfi_eom");
        mlfi_cleanup(ctx, MLFIABORT);
        return SMFIS_TEMPFAIL;
    }

    DEBUGLOG("mlfi_eom start");

    if (priv->mlfi_savebcc != NULL) {
	for (i = 0; (priv->mlfi_savebcc + i)->rcpt_addr != NULL; i++) {
	    if (smfi_addrcpt(ctx, (priv->mlfi_savebcc + i)->rcpt_addr) != MI_SUCCESS) {
		log(ERR_AUTOBCC, "mlfi_eom", "smfi_addrcpt() failed.");
		mlfi_cleanup(ctx, MLFIABORT);
		return SMFIS_TEMPFAIL;
	    }
	}
    }

    if (priv->mlfi_encstatus == NOT_ENC) {
	mlfi_cleanup(ctx, MLFIABORT);
	return SMFIS_ACCEPT;
    }

    /* Encryption Process */
    ret = zip_convert_mail(ctx, priv->mlfi_minfo, priv->mlfi_conf, 
			   priv->mlfi_savefrom, priv->mlfi_passlist, 
		 	   priv->mlfi_rdmpasslist);

    /* Reset MailSave Information */
    mailsave_reset(priv->mlfi_minfo);
    mlfi_cleanup(ctx, MLFIABORT);

    if (ret == PM_FAILED) {
        return SMFIS_TEMPFAIL;
    } else if (ret == ZIP_CONVERT_ACCEPT) {
        return SMFIS_ACCEPT;
    } else if (ret == ZIP_CONVERT_INVALID_FILENAME) {
        if (smfi_setreply(ctx, CODE_INVALID_FILENAME, XCODE_INVALID_FILENAME,
                          MESSAGE_INVALID_FILENAME) != MI_SUCCESS) {
            return SMFIS_TEMPFAIL;
        }
	return SMFIS_REJECT;
    }

    return SMFIS_CONTINUE;
}

/*
 * mlfi_close
 * close
 */
sfsistat
mlfi_close(SMFICTX *ctx)
{
    DEBUGLOG("mlfi_close start");

    mlfi_cleanup(ctx, MLFICLOSE);

    return SMFIS_CONTINUE;
}

/*
 * mlfi_abort
 * abort
 */
sfsistat
mlfi_abort(SMFICTX *ctx)
{
    DEBUGLOG("mlfi_abort start");

    mlfi_cleanup(ctx, MLFIABORT);
    return SMFIS_CONTINUE;
}

/*
 * mlfi_cleanup
 * cleanup
 */
sfsistat
mlfi_cleanup(SMFICTX *ctx, int flag)
{
    sfsistat rstat = SMFIS_CONTINUE;
    struct mlfiPriv *priv;
    /* null data return */
    if ((priv = MLFIPRIV(ctx)) == NULL) {
        return rstat;
    }

    /* priv release */
    if (priv->mlfi_savefrom != NULL) {
        free(priv->mlfi_savefrom);
	priv->mlfi_savefrom = NULL;
    }

    if (priv->mlfi_savercpt != NULL) {
        free_rcptlist(priv->mlfi_savercpt);
	priv->mlfi_savercpt = NULL;
    }

    if (priv->mlfi_passlist != NULL) {
        free_rcptinfo(priv->mlfi_passlist);
	priv->mlfi_passlist = NULL;
    }

    if (priv->mlfi_rdmpasslist != NULL) {
        free_rcptinfo(priv->mlfi_rdmpasslist);
	priv->mlfi_rdmpasslist = NULL;
    }

    if (priv->mlfi_savebcc != NULL) {
	free_rcptlist(priv->mlfi_savebcc);
	priv->mlfi_savebcc = NULL;
    }

    if (flag == MLFICLOSE) {
        /* config relase */
        if (priv->mlfi_conf != NULL) {
            config_release(priv->mlfi_conf);
	    priv->mlfi_conf = NULL;
        }

	if (priv->mlfi_minfo != NULL) {
	    mailsave_clean(priv->mlfi_minfo);
	    free(priv->mlfi_minfo);
	    priv->mlfi_minfo = NULL;
	}

	free(priv);

	/* set NULL */
	smfi_setpriv(ctx, NULL);
    }

    return rstat;
}

static void *
call_command(void *arg)
{
    struct thread_control *tcl;
    int ret = 0;

    tcl = (struct thread_control *)arg;

    if (accept_command(tcl->configname , tcl->addr, tcl->port) != 0) {
	log(ERR_RELOAD_SERVER, "call_command");
    }

    pthread_exit(&ret);
}

/*
 * usage
 * 
 */
void
usage(char *arg)
{
    fprintf(stderr, "usage: %s [-t timeout] [config file]\n", arg);
}

int
main(int argc, char *argv[])
{

    char  *args = "t:";
    char  *confname;
    char  *tmp;
    char   c;
    char  oconn[OCONN_LENGTH];
    int    timeout = DEFAULT_TIMEOUT;
    unsigned long tmp_val;
    struct thread_control tcl;
    struct config *cfg;

    /* set rand function */
    srand48(time(NULL));

    /* ignore sigpipe signal */
    signal(SIGPIPE, SIG_IGN); 

    /* set error output to stderr */
    init_log();

    /* arg */
    while ((c = getopt(argc, argv, args)) != -1) {

        switch (c) {

        /* arg time out */
        case 't':
            if((optarg == NULL) || (optarg[0] == '\0')) {
                usage(argv[0]);
                exit(1);
            }

            tmp_val = strtoul(optarg, &tmp, 10);
            if((*tmp != '\0') ||
               ((tmp_val == ULONG_MAX) && (errno == ERANGE)) ||
	       (tmp_val > INT_MAX)) {
                usage(argv[0]);
                exit(1);
            }

            timeout = (int) tmp_val;
            break;

        /* error */
        default:
            usage(argv[0]);
            exit(1);
        }
    }

    /* check arg */
    if (argc > MAX_ARGS) {
	usage(argv[0]);
	exit(1);
    } else if(optind + 1 == argc) {

	/* set config name */
	confname = argv[optind];
    } else if (optind == argc) {
	/* set default config name */
	confname = DEFAULT_CONFFILE;
    } else {
	usage(argv[0]);
	exit(1);
    }

    /* read config file */
    if (reload_config(confname) != 0) {
	fprintf(stderr, STDERR_CONFIG_READ, confname);
	fprintf(stderr, "\n");
        exit(2);
    }

    cfg = config_init();

    /* set thread data */
    tcl.configname = confname;
    tcl.port = cfg->cf_commandport;
    tcl.addr = strdup(cfg->cf_listenip);
    if (tcl.addr == NULL) {
	fprintf(stderr, STDERR_MEMORY_ALLOCATE, "tcl.addr", strerror(errno));
	fprintf(stderr, "\n");
	exit(2);
    }

    /* initialize GMime */
    g_mime_init (GMIME_ENABLE_RFC2047_WORKAROUNDS);

    /* prepare socket string */
    sprintf(oconn, OCONN, cfg->cf_listenport, cfg->cf_listenip);

    /* config relase */
    config_release(cfg);

    /* command line thread */
    pthread_create(&child, NULL, call_command, &tcl);

    /* set socket */
    if(smfi_setconn(oconn) == MI_FAILURE) {
        fprintf(stderr, ERR_MILTER_SET_SOCKET, strerror(errno));
	fprintf(stderr, "\n");
	g_mime_shutdown();
        exit(3);
    }

    /* set time out */
    if(smfi_settimeout(timeout) == MI_FAILURE) {
        fprintf(stderr, ERR_MILTER_SET_TIMEOUT, strerror(errno));
	fprintf(stderr, "\n");
	g_mime_shutdown();
        exit(3);
    }

    /* set register */
    smfilter.xxfi_flags |= SMFIF_ADDRCPT;
    if(smfi_register(smfilter) == MI_FAILURE) {
        fprintf(stderr, ERR_MILTER_REGISTER, strerror(errno));
	fprintf(stderr, "\n");
	g_mime_shutdown();
        exit(4);
    }

    /* run main */
    if(smfi_main() == MI_FAILURE) {
        fprintf(stderr, ERR_MILTER_START, strerror(errno));
	fprintf(stderr, "\n");
	g_mime_shutdown();
        exit(5);
    }

    g_mime_shutdown();

    exit(0);
}
